@@ -1,13 +1,20 @@
-import json
-import os
-import warnings
+"""
+Utilities for the HDF5 I/O backend,
+e.g., for wrapping HDF5 datasets on read, wrapping arrays for configuring write, or
+writing the spec among others"""
+
+from collections import deque
 from abc import ABCMeta, abstractmethod
 from collections.abc import Iterable
 from copy import copy
 
-import numpy as np
 from h5py import Group, Dataset, RegionReference, Reference, special_dtype
 from h5py import filters as h5py_filters
+import json
+import numpy as np
+import warnings
+import os
+import logging
 
 from ...array import Array
 from ...data_utils import DataIO, AbstractDataChunkIterator
@@ -17,6 +24,66 @@ from ...spec import SpecWriter, SpecReader
 from ...utils import docval, getargs, popargs, call_docval_func, get_docval
 
 
+class HDF5IODataChunkIteratorQueue(deque):
+    """
+    Helper class used by HDF5IO to manage the write for DataChunkIterators
+
+    Each queue element must be a tuple of two elements:
+    1) the dataset to write to and 2) the AbstractDataChunkIterator with the data
+    """
+    def __init__(self):
+        self.logger = logging.getLogger('%s.%s' % (self.__class__.__module__, self.__class__.__qualname__))
+        super().__init__()
+
+    @classmethod
+    def _write_chunk(cls, dset, data):
+        """
+        Read a chunk from the given DataChunkIterator and write it to the given Dataset
+
+        :param dset: The Dataset to write to
+        :type dset: Dataset
+        :param data: The DataChunkIterator to read from
+        :type data: AbstractDataChunkIterator
+        :return: True if a chunk was written, False otherwise
+        :rtype: bool
+
+        """
+        # Read the next data block
+        try:
+            chunk_i = next(data)
+        except StopIteration:
+            return False
+        # Determine the minimum array size required to store the chunk
+        max_bounds = chunk_i.get_min_bounds()
+        # Expand the dataset if needed
+        dset.id.extend(max_bounds)
+        # Write the data
+        dset[chunk_i.selection] = chunk_i.data
+
+        return True
+
+    def exhaust_queue(self):
+        """
+        Read and write from any queued DataChunkIterators in a round-robin fashion
+        """
+        while len(self) > 0:
+            self.logger.debug("Exhausting DataChunkIterator from queue (length %d)" % len(self))
+            dset, data = self.popleft()
+            if self._write_chunk(dset, data):
+                self.append(dataset=dset, data=data)
+
+    def append(self, dataset, data):
+        """
+        Append a value to the queue
+
+        :param dataset: The dataset where the DataChunkIterator is written to
+        :type dataset: Dataset
+        :param data: DataChunkIterator with the data to be written
+        :type data: AbstractDataChunkIterator
+        """
+        super().append((dataset, data))
+
+
 class H5Dataset(HDMFDataset):
     @docval({'name': 'dataset', 'type': (Dataset, Array), 'doc': 'the HDF5 file lazily evaluate'},
             {'name': 'io', 'type': 'HDF5IO', 'doc': 'the IO object that was used to read the underlying dataset'})
@@ -461,7 +528,7 @@ class H5DataIO(DataIO):
 
     def get_io_params(self):
         """
-        Returns a dict with the I/O parameters specifiedin in this DataIO.
+        Returns a dict with the I/O parameters specified in this DataIO.
         """
         ret = dict(self.__iosettings)
         ret['link_data'] = self.__link_data
