@@ -15,6 +15,14 @@ __macros = {
     'data': []
 }
 
+try:
+    # optionally accept zarr.Array as array data to support conversion of data from Zarr to HDMF
+    import zarr
+    __macros['array_data'].append(zarr.Array)
+except ImportError:
+    pass
+
+
 # code to signify how to handle positional arguments in docval
 AllowPositional = Enum('AllowPositional', 'ALLOWED WARNING ERROR')
 
@@ -230,10 +238,11 @@ def __parse_args(validator, args, kwargs, enforce_type=True, enforce_shape=True,
 
         if args:
             if allow_positional == AllowPositional.WARNING:
-                msg = 'Positional arguments are discouraged and may be forbidden in a future release.'
+                msg = ('Using positional arguments for this method is discouraged and will be deprecated '
+                       'in a future major release. Please use keyword arguments to ensure future compatibility.')
                 future_warnings.append(msg)
             elif allow_positional == AllowPositional.ERROR:
-                msg = 'Only keyword arguments (e.g., func(argname=value, ...)) are allowed.'
+                msg = 'Only keyword arguments (e.g., func(argname=value, ...)) are allowed for this method.'
                 syntax_errors.append(msg)
 
         # iterate through the docval specification and find a matching value in args / kwargs
@@ -297,6 +306,7 @@ def __parse_args(validator, args, kwargs, enforce_type=True, enforce_shape=True,
             arg = next(it)
 
         # process arguments of the docval specification with a default value
+        # NOTE: the default value will be deepcopied, so 'default': list() is safe unlike in normal python
         while True:
             argname = arg['name']
             if argname in kwargs:
@@ -723,6 +733,27 @@ def popargs(*argnames):
     return ret
 
 
+def popargs_to_dict(keys, argdict):
+    """Convenience function to retrieve and remove arguments from a dictionary in batch into a dictionary.
+
+    Same as `{key: argdict.pop(key) for key in keys}` with a custom ValueError
+
+    :param keys: Iterable of keys to pull out of argdict
+    :type keys: Iterable
+    :param argdict: Dictionary to process
+    :type dict: dict
+    :raises ValueError: if an argument name is not found in the dictionary
+    :return: a dict of arguments removed
+    """
+    ret = dict()
+    for arg in keys:
+        try:
+            ret[arg] = argdict.pop(arg)
+        except KeyError as ke:
+            raise ValueError('Argument not found in dict: %s' % str(ke))
+    return ret
+
+
 class ExtenderMeta(ABCMeta):
     """A metaclass that will extend the base class initialization
        routine by executing additional functions defined in
